<?PHP

defined('COT_CODE') or die("Wrong URL.");

/**
 * Empty initialization function.
 * Used to provide client with XSS key and other required variables.
 * 
 * @return string JSON encoded data
 */
function api_init()
{
	return api_response();
}

/**
 * Verifies incoming API requests by checking the provided signature.
 */
function verify_signature()
{
	global $cfg;
	
	$app_id = cot_import('app_id', 'R', 'TXT');
	$signature = cot_import('signature', 'R', 'ALP', 40);
		
	$calc_signature = ($_SERVER['REQUEST_METHOD'] == 'GET') ?
		calculate_signature($_GET) : calculate_signature($_POST);
	if ($app_id != $cfg['app_id'] || $signature !== $calc_signature)
	{
		global $env;
		$env['status'] = '900 Invalid Signature';
		die(api_response());
	}
}

/**
 * Verifies the used request method to be of correct type.
 *
 * @param string $method Accepted HTTP Request Method, 'GET' or 'POST'
 */
function verify_method($method)
{
	if ($_SERVER['REQUEST_METHOD'] != $method)
	{
		global $env;
		$env['status'] = '405 Method Not Allowed';
		die(api_response());
	}
}

/**
 * Handles outgoing API data responses.
 * Will remove empty values and calculate the signature.
 * 
 * @param array $data Response data formatted as associative array
 * @return string JSON encoded data
 */
function api_response($data = array())
{
	global $cfg, $env, $sys, $api_debug;
	if (!is_array($data) || (is_array($data) && !array_key_exists('data', $data)))
	{
		$data = array('data' => $data);
	}
	
	$data['app_id'] = $cfg['app_id'];
	$data['timestamp'] = $sys['now'];
	$data['status_code'] = get_status('code');
	$data['status_text'] = get_status('text');
	if (is_array($data['data']) && isset($data['data']['xk']))
	{
		$data['xk'] = $data['data']['xk'];
		unset($data['data']['xk']);
	}
	else
	{
		$data['xk'] = $sys['xk'];
	}
	$data = array_filter($data);
	$data['signature'] = calculate_signature($data);
	
	ksort($data);
	
	if ($api_debug)
	{
		$data['debug'] = array(
			'GET' => $_GET, 
			'POST' => $_POST
		);
	}
	
	return json_encode($data);
}

/**
 * Calculate SHA-1 signature. Sorts data by key before encoding.
 * Ignores keys 'signature', 'm' and 'a'.
 *
 * @param array $data Data formatted as associative array
 * @return string SHA-1 signature
 */
function calculate_signature($data)
{
	$res = '';
	ksort($data);
	foreach ($data as $key => $val)
	{
		if ($key == 'signature' || $key == 'm' || $key == 'a') continue;
		$res .= $key.$val;
	}
	return sha1($res);
}

/**
 * Get code or text part of the environment (HTTP) status code
 *
 * @param string $type 'code' or 'text'
 * @return string
 */
function get_status($type = 'code')
{
	global $env;
	$status = trim($env['status']);
	if ($type == 'code')
	{
		$res = trim(substr($status, 0, strpos($status, ' ')));
		if(!$res) $res = '200';
	}
	else
	{
		$res = trim(substr($status, strpos($status, ' ')));
		if(!$res) $res = 'OK';
	}
	return $res;
}

?>